Method: EventMachine.connection_count
- Defined in:
- lib/eventmachine.rb
.connection_count ⇒ Integer
Returns the total number of connections (file descriptors) currently held by the reactor. Note that a tick must pass after the 'initiation' of a connection for this number to increment. It's usually accurate, but don't rely on the exact precision of this number unless you really know EM internals.
945 946 947 |
# File 'lib/eventmachine.rb', line 945 def self.connection_count self.get_connection_count end |