Method: EventMachine.reactor_thread
- Defined in:
- lib/eventmachine.rb
.reactor_thread ⇒ Thread (readonly)
Exposed to allow joining on the thread, when run in a multithreaded environment. Performing other actions on the thread has undefined semantics (read: a dangerous endevor).
79 80 81 |
# File 'lib/eventmachine.rb', line 79 def reactor_thread @reactor_thread end |